@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 from translate.lang import factory
@@ -35,7 +34,7 @@ def test_getlanguage():
     assert language.fullname == 'Icelandic'
 
     language = factory.getlanguage('or')
-    assert "startcaps" in language.ignoretests
+    assert "startcaps" in language.ignoretests['all']
 
     #Test with a language code contains '@'
     language = factory.getlanguage('ca@valencia')
