@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2006,2008-2010 Zuza Software Foundation
@@ -26,6 +25,7 @@ for examples and usage instructions.
 
 import os
 import re
+import six
 from hashlib import md5
 
 from translate.convert import dtd2po
@@ -42,6 +42,7 @@ def add_prefix(prefix, stringelems):
                 break
     return stringelems
 
+
 podebug_parsers = general.parsers
 podebug_parsers.remove(general.CapsPlaceable.parse)
 podebug_parsers.remove(general.CamelCasePlaceable.parse)
@@ -49,13 +50,14 @@ podebug_parsers.remove(general.CamelCasePlaceable.parse)
 
 class podebug:
 
-    def __init__(self, format=None, rewritestyle=None, ignoreoption=None):
+    def __init__(self, format=None, rewritestyle=None, ignoreoption=None, preserveplaceholders=False):
         if format is None:
             self.format = ""
         else:
             self.format = format
         self.rewritefunc = getattr(self, "rewrite_%s" % rewritestyle, None)
         self.ignorefunc = getattr(self, "ignore_%s" % ignoreoption, None)
+        self.preserveplaceholders = preserveplaceholders
 
     def apply_to_translatables(self, string, func):
         """Applies func to all translatable strings in string."""
@@ -73,11 +75,11 @@ class podebug:
         if not isinstance(string, StringElem):
             string = StringElem(string)
         string.sub.insert(0, prepend)
-        if unicode(string).endswith(u'\n'):
+        if six.text_type(string).endswith(u'\n'):
             # Try and remove the last character from the tree
             try:
                 lastnode = string.flatten()[-1]
-                if isinstance(lastnode.sub[-1], unicode):
+                if isinstance(lastnode.sub[-1], six.text_type):
                     lastnode.sub[-1] = lastnode.sub[-1].rstrip(u'\n')
             except IndexError:
                 pass
@@ -137,6 +139,33 @@ class podebug:
             self.apply_to_translatables(string, lambda s: re.sub(a, b, s))
         return string
 
+    PRESERVE_PLACEABLE_PARSERS = [
+        general.UrlPlaceable.parse,
+        general.EmailPlaceable.parse,
+        general.XMLTagPlaceable.parse,
+        general.DoubleAtPlaceable.parse,
+        general.BracePlaceable.parse,
+        general.PythonFormattingPlaceable.parse
+    ]
+    # These parsers extract placeholders that should NOT be transformed during character-level rewrites
+    # when the preserveplaceholders flag is True. It is not the full set of placeable parsers available
+    # as some of them are not appropriate for this usage.
+
+    def transform_characters_preserving_placeholders(self, s, transform):
+        rich_string = rich_parse(s, self.PRESERVE_PLACEABLE_PARSERS)
+        string_elements = rich_string.depth_first(filter=lambda e: e.isleaf())
+
+        transformed = []
+
+        for element in string_elements:
+            if element.istranslatable:
+                for character in str(element):
+                    transformed.append(transform(character))
+            else:
+                transformed.append(element.sub[0])
+
+        return u''.join(transformed)
+
     REWRITE_UNICODE_MAP = u"ȦƁƇḒḖƑƓĦĪĴĶĿḾȠǾƤɊŘŞŦŬṼẆẊẎẐ" + u"[\\]^_`" + u"ȧƀƈḓḗƒɠħīĵķŀḿƞǿƥɋřşŧŭṽẇẋẏẑ"
 
     def rewrite_unicode(self, string):
@@ -151,13 +180,18 @@ class podebug:
             return self.REWRITE_UNICODE_MAP[loc]
 
         def transformer(s):
-            return ''.join([transpose(c) for c in s])
+            if self.preserveplaceholders:
+                return self.transform_characters_preserving_placeholders(s, transpose)
+            else:
+                return ''.join([transpose(c) for c in s])
+
         self.apply_to_translatables(string, transformer)
         return string
 
-    REWRITE_FLIPPED_MAP = u"¡„#$%⅋,()⁎+´-˙/012Ɛᔭ59Ƚ86:;<=>¿@" + \
-            u"∀ԐↃᗡƎℲ⅁HIſӼ⅂WNOԀÒᴚS⊥∩ɅＭX⅄Z" + u"[\\]ᵥ_," + \
-            u"ɐqɔpǝɟƃɥıɾʞʅɯuodbɹsʇnʌʍxʎz"
+    REWRITE_FLIPPED_MAP = (
+        u"¡„#$%⅋,()⁎+´-˙/012Ɛᔭ59Ƚ86:;<=>¿@" +
+        u"∀ԐↃᗡƎℲ⅁HIſӼ⅂WNOԀÒᴚS⊥∩ɅＭX⅄Z" +
+        u"[\\]ᵥ_," + u"ɐqɔpǝɟƃɥıɾʞʅɯuodbɹsʇnʌʍxʎz")
     # Brackets should be swapped if the string will be reversed in memory.
     # If a right-to-left override is used, the brackets should be
     # unchanged.
@@ -180,7 +214,10 @@ class podebug:
             return self.REWRITE_FLIPPED_MAP[loc]
 
         def transformer(s):
-            return u"\u202e" + u''.join([transpose(c) for c in s])
+            if self.preserveplaceholders:
+                return u"\u202e" + self.transform_characters_preserving_placeholders(s, transpose)
+            else:
+                return u"\u202e" + u''.join([transpose(c) for c in s])
             # To reverse instead of using the RTL override:
             #return u''.join(reversed([transpose(c) for c in s]))
         self.apply_to_translatables(string, transformer)
@@ -201,7 +238,7 @@ class podebug:
         return False
 
     def ignore_libreoffice(self, unit):
-        return ignore_openoffice(unit)
+        return self.ignore_openoffice(unit)
 
     def ignore_mozilla(self, unit):
         locations = unit.getlocations()
@@ -235,7 +272,7 @@ class podebug:
                 hashable = unit.getlocations()[0]
             else:
                 hashable = unit.source
-            prefix = prefix.replace("@hash_placeholder@", md5(hashable).hexdigest()[:self.hash_len])
+            prefix = prefix.replace("@hash_placeholder@", md5(hashable.encode('utf-8')).hexdigest()[:self.hash_len])
         if unit.istranslated():
             rich_string = unit.rich_target
         else:
@@ -269,7 +306,7 @@ class podebug:
                 formatted = os.path.dirname(store.filename)
             elif formatstr.endswith("h"):
                 try:
-                    self.hash_len = int(filter(str.isdigit, formatstr[1:-1]))
+                    self.hash_len = int(''.join([c for c in formatstr[1:-1] if c.isdigit()]))
                 except ValueError:
                     self.hash_len = 4
                 formatted = "@hash_placeholder@"
@@ -278,8 +315,8 @@ class podebug:
             formatoptions = formatstr[1:-1]
             if formatoptions and not formatstr.endswith("h"):
                 if "c" in formatoptions and formatted:
-                    formatted = formatted[0] + filter(lambda x: x.lower() not in "aeiou", formatted[1:])
-                length = filter(str.isdigit, formatoptions)
+                    formatted = formatted[0] + ''.join([c for c in formatted[1:] if c.lower() not in "aeiou"])
+                length = ''.join([c for c in formatoptions if c.isdigit()])
                 if length:
                     formatted = formatted[:int(length)]
             prefix = prefix.replace(formatstr, formatted)
@@ -306,38 +343,51 @@ class podebug:
         return dirshrunk + baseshrunk
 
 
-def convertpo(inputfile, outputfile, templatefile, format=None, rewritestyle=None, ignoreoption=None):
+def convertpo(inputfile, outputfile, templatefile, format=None, rewritestyle=None, ignoreoption=None,
+              preserveplaceholders=None):
     """Reads in inputfile, changes it to have debug strings, writes to outputfile."""
     # note that templatefile is not used, but it is required by the converter...
     inputstore = factory.getobject(inputfile)
     if inputstore.isempty():
         return 0
-    convertor = podebug(format=format, rewritestyle=rewritestyle, ignoreoption=ignoreoption)
+    convertor = podebug(format=format,
+                        rewritestyle=rewritestyle,
+                        ignoreoption=ignoreoption,
+                        preserveplaceholders=preserveplaceholders)
     outputstore = convertor.convertstore(inputstore)
-    outputfile.write(str(outputstore))
+    outputstore.serialize(outputfile)
     return 1
 
 
 def main():
     from translate.convert import convert
     formats = {
-            "po": ("po", convertpo), "pot": ("po", convertpo),
-            "xlf": ("xlf", convertpo),
-            "tmx": ("tmx", convertpo),
+        "po": ("po", convertpo), "pot": ("po", convertpo),
+        "xlf": ("xlf", convertpo),
+        "tmx": ("tmx", convertpo),
     }
     parser = convert.ConvertOptionParser(formats, description=__doc__)
     # TODO: add documentation on format strings...
-    parser.add_option("-f", "--format", dest="format", default="",
+    parser.add_option(
+        "-f", "--format", dest="format", default="",
         help="specify format string")
-    parser.add_option("", "--rewrite", dest="rewritestyle",
+    parser.add_option(
+        "", "--rewrite", dest="rewritestyle",
         type="choice", choices=podebug.rewritelist(), metavar="STYLE",
         help="the translation rewrite style: %s" % ", ".join(podebug.rewritelist()))
-    parser.add_option("", "--ignore", dest="ignoreoption",
+    parser.add_option(
+        "", "--ignore", dest="ignoreoption",
         type="choice", choices=podebug.ignorelist(), metavar="APPLICATION",
         help="apply tagging ignore rules for the given application: %s" % ", ".join(podebug.ignorelist()))
+    parser.add_option("", "--preserveplaceholders", dest="preserveplaceholders",
+                      default=False, action="store_true",
+                      help="attempt to exclude characters that are part of placeholders when performing character-level"
+                           " rewrites so that consuming applications can still use the placeholders to generate final "
+                           "output")
     parser.passthrough.append("format")
     parser.passthrough.append("rewritestyle")
     parser.passthrough.append("ignoreoption")
+    parser.passthrough.append("preserveplaceholders")
     parser.run()
 
 
